$def with (channels, entries, cards, events, reviews, rooms, media)
$var title: Content

<h2>Channels</h2>
<p><a href=/pub/channels>$len(channels) total</a></p>
<ul>
$for channel in channels:
    <li><a href=/pub/channels/$channel>$channel</a></li>
</ul>

<h2>Entries</h2>
<p><a href=/pub/entries>$len(entries) total</a></p>

<h2>Cards</h2>
<p><a href=/pub/cards>$len(cards) total</a></p>

<h2>Events</h2>
<p><a href=/pub/events>$len(events) total</a></p>

<h2>Reviews</h2>
<p><a href=/pub/reviews>$len(reviews) total</a></p>

<h2>Rooms</h2>
<p><a href=/pub/rooms>$len(rooms) total</a></p>

<h2>Media</h2>
<p><a href=/pub/media>$len(media) total</a></p>
